# disai_jazz/utils.py
class InputType:
    def __init__(self, input_type):
        self.input_type = input_type

    def __str__(self):
        return self.input_type

class OutputType:
    def __init__(self, output_type):
        self.output_type = output_type

    def __str__(self):
        return self.output_type
